from __future__ import annotations

from typing import TYPE_CHECKING

from typing_extensions import override

from great_expectations_cloud.agent.actions.agent_action import (
    ActionResult,
    AgentAction,
)
from great_expectations_cloud.agent.event_handler import register_event_action
from great_expectations_cloud.agent.models import (
    CreatedResource,
    RunCheckpointEvent,
    RunScheduledCheckpointEvent,
)

if TYPE_CHECKING:
    from great_expectations.data_context import CloudDataContext


class RunCheckpointAction(AgentAction[RunCheckpointEvent]):
    # TODO: New actions need to be created that are compatible with GX v1 and registered for v1.
    #  This action is registered for v0, see register_event_action()

    @override
    def run(self, event: RunCheckpointEvent, id: str) -> ActionResult:
        return run_checkpoint(self._context, event, id)


def run_checkpoint(
    context: CloudDataContext,
    event: RunCheckpointEvent | RunScheduledCheckpointEvent,
    id: str,
) -> ActionResult:
    """Note: the logic for this action is broken out into this function so that
    the same logic can be used for both RunCheckpointEvent and RunScheduledCheckpointEvent."""
    # TODO: move connection testing into OSS; there isn't really a reason it can't be done there
    for datasource_name, data_asset_names in event.datasource_names_to_asset_names.items():
        datasource = context.get_datasource(datasource_name)
        datasource.test_connection(test_assets=False)  # raises `TestConnectionError` on failure
        for (
            data_asset_name
        ) in data_asset_names:  # only test connection for assets that are validated in checkpoint
            asset = datasource.get_asset(data_asset_name)
            asset.test_connection()  # raises `TestConnectionError` on failure
    checkpoint_run_result = context.run_checkpoint(
        ge_cloud_id=event.checkpoint_id,
        batch_request={"options": event.splitter_options} if event.splitter_options else None,
    )

    validation_results = checkpoint_run_result.run_results
    created_resources = []
    for key in validation_results.keys():
        created_resource = CreatedResource(
            resource_id=validation_results[key]["actions_results"]["store_validation_result"]["id"],
            type="SuiteValidationResult",
        )
        created_resources.append(created_resource)

    return ActionResult(
        id=id,
        type=event.type,
        created_resources=created_resources,
    )


register_event_action("0", RunCheckpointEvent, RunCheckpointAction)



from __future__ import annotations

import logging
from typing import Final

from typing_extensions import override

from great_expectations_cloud.agent.actions.agent_action import (
    ActionResult,
    AgentAction,
)
from great_expectations_cloud.agent.agent_warnings import warn_unknown_event
from great_expectations_cloud.agent.models import (
    UnknownEvent,
)

LOGGER: Final[logging.Logger] = logging.getLogger(__name__)


class UnknownEventAction(AgentAction[UnknownEvent]):
    @override
    def run(self, event: UnknownEvent, id: str) -> ActionResult:
        warn_unknown_event(event.type)
        # noop
        return ActionResult(
            id=id,
            type=event.type,
            created_resources=[],
        )



from __future__ import annotations

import enum
from datetime import datetime
from typing import TYPE_CHECKING

from great_expectations.datasource.fluent import Datasource as FluentDatasource
from great_expectations.exceptions import DataContextError

from great_expectations_cloud.agent.actions.agent_action import (
    ActionResult,
)
from great_expectations_cloud.agent.models import (
    CreatedResource,
    RunDataAssistantEvent,
)

if TYPE_CHECKING:
    from great_expectations.core import ExpectationSuite
    from great_expectations.core.batch import BatchRequest
    from great_expectations.data_context.data_context import CloudDataContext
    from great_expectations.rule_based_profiler.data_assistant_result.data_assistant_result import (
        DataAssistantResult,
    )


class DataAssistantType(str, enum.Enum):
    MISSINGNESS = "Missingness"
    ONBOARDING = "Onboarding"


def build_batch_request(
    context: CloudDataContext,
    event: RunDataAssistantEvent,
) -> BatchRequest:
    datasource = context.get_datasource(datasource_name=event.datasource_name)
    if not isinstance(datasource, FluentDatasource):
        raise TypeError(  # noqa: TRY003 # one off error
            "The RunDataAssistant Action can only be used with a fluent-style Data Source."
        )

    asset = datasource.get_asset(asset_name=event.data_asset_name)
    try:
        batch_request = asset.build_batch_request()
    except ValueError as e:
        raise ValueError(  # noqa: TRY003 # one off error
            "The RunDataAssistant Action for data assistant cannot be used with an "
            "in-memory Data Asset."
        ) from e

    return batch_request


def build_action_result(
    context: CloudDataContext,
    data_assistant_name: DataAssistantType,
    event: RunDataAssistantEvent,
    data_assistant_result: DataAssistantResult,
    id: str,
) -> ActionResult:
    if event.expectation_suite_name is not None:
        expectation_suite_name = event.expectation_suite_name
    else:
        expectation_suite_name = f"{event.data_asset_name} {data_assistant_name.value} Suite"

    checkpoint_name = f"{event.data_asset_name} {data_assistant_name.value} Checkpoint"

    # build tz aware timestamp
    tz = datetime.now().astimezone().tzinfo
    timestamp = datetime.now(tz=tz)

    # ensure we have unique names for created resources
    try:
        context.get_expectation_suite(expectation_suite_name=expectation_suite_name)
        # if that didn't error, this name exists, so we add the timestamp
        expectation_suite_name = f"{expectation_suite_name} {timestamp}"
    except DataContextError:
        # resource is unique
        pass

    try:
        context.get_checkpoint(name=checkpoint_name)
        # if that didn't error, this name exists, so we add the timestamp
        checkpoint_name = f"{checkpoint_name} {timestamp}"
    except DataContextError:
        # resource is unique
        pass

    expectation_suite: ExpectationSuite = data_assistant_result.get_expectation_suite(
        expectation_suite_name=expectation_suite_name
    )
    context.add_expectation_suite(expectation_suite=expectation_suite)

    checkpoint = context.add_checkpoint(
        name=checkpoint_name,
        config_version=1.0,
        class_name="Checkpoint",
        validations=[
            {
                "expectation_suite_name": expectation_suite_name,
                "expectation_suite_ge_cloud_id": expectation_suite.ge_cloud_id,
                "batch_request": {
                    "datasource_name": event.datasource_name,
                    "data_asset_name": event.data_asset_name,
                },
            }
        ],
    )

    expectation_suite_id = expectation_suite.ge_cloud_id
    checkpoint_id = checkpoint.ge_cloud_id
    if expectation_suite_id is None or checkpoint_id is None:
        raise ValueError("Cloud backed resources must have an ID.")  # noqa: TRY003 # one off error
    return ActionResult(
        id=id,
        type=event.type,
        created_resources=[
            CreatedResource(resource_id=expectation_suite_id, type="ExpectationSuite"),
            CreatedResource(resource_id=checkpoint_id, type="Checkpoint"),
        ],
    )



from __future__ import annotations

from great_expectations.data_context.cloud_constants import CLOUD_DEFAULT_BASE_URL
from pydantic.v1 import AnyUrl, BaseSettings, ValidationError


class GxAgentEnvVars(BaseSettings):
    # pydantic will coerce this string to AnyUrl type
    gx_cloud_base_url: AnyUrl = CLOUD_DEFAULT_BASE_URL  # type: ignore[assignment]
    gx_cloud_organization_id: str
    gx_cloud_access_token: str

    def __init__(self, **overrides: str | AnyUrl) -> None:
        """
        Custom __init__ to prevent type error when relying on environment variables.

        TODO:once mypy fully support annoting **kwargs with a Unpack[TypedDict], we should do that.
        https://peps.python.org/pep-0692/
        """
        super().__init__(**overrides)


def generate_config_validation_error_text(validation_error: ValidationError) -> str:
    missing_variables = ", ".join(
        [str(validation_error["loc"][0]) for validation_error in validation_error.errors()]
    )
    error_text = f"Missing or badly formed environment variable(s). Make sure to set the following environment variable(s): {missing_variables}"
    return error_text
